from typing import Any

from requests import Response, Session


class HartwareAuthApiSession(Session):
    def __init__(self, auth_url: str):
        super(HartwareAuthApiSession, self).__init__()
        self.auth_url = auth_url

        self.access_token = ""
        self.refresh_token = ""

    def login(self, username, password) -> str:
        response = self.post(
            f"{self.auth_url}/api/v1/login/",
            data={"username": username, "password": password},
        )
        response.raise_for_status()

        self.access_token = response.json()["access_token"]
        self.refresh_token = response.json()["refresh_token"]

        self.headers["Authorization"] = f"Bearer {self.access_token}"

        return self.access_token

    def refresh(self) -> str:
        response = self.post(
            f"{self.auth_url}/api/v1/login/refresh/",
            headers={"Authorization": f"Bearer {self.refresh_token}"},
        )
        response.raise_for_status()

        self.access_token = response.json()["access_token"]

        self.headers["Authorization"] = f"Bearer {self.access_token}"

        return self.access_token

    def get_user(self) -> dict:
        response = self.get(f"{self.auth_url}/api/v1/users/me/")
        response.raise_for_status()

        return response.json()


class HartwareApiSession(Session):
    def __init__(self, base_url: str, auth_session: HartwareAuthApiSession):
        super(HartwareApiSession, self).__init__()

        self.base_url = base_url
        self.auth_session = auth_session

    def setup_headers(self) -> None:
        if authorization_header := self.auth_session.headers.get("Authorization"):
            self.headers["Authorization"] = authorization_header

    def request(self, method: str, url: str, **kwargs: Any) -> Response:  # type: ignore
        url = f"{self.base_url}/{url}"

        if not self.headers.get("Authorization"):
            self.setup_headers()

        response = super().request(method, url, **kwargs)

        if response.status_code == 401:
            self.auth_session.access_token = ""
            self.auth_session.refresh()

            self.setup_headers()

            response = super().request(method, url, **kwargs)

        if not response.status_code == 200:
            response.raise_for_status()

        return response



class CustomFieldType:
    @classmethod
    def validate(cls): ...

    @classmethod
    def __get_validators__(cls):
        yield cls.validate


class BooleanFromString(CustomFieldType):
    @classmethod
    def validate(cls, v):
        if isinstance(v, bool):  # Let pass boolean default values
            return v

        if v.lower() in ("yes", "true", "1"):
            return True
        elif v.lower() in ("no", "false", "0"):
            return False
        else:
            raise ValueError(
                "Invalid string representation of a boolean, should be either 1|0|true|false|yes|no"
            )



import logging
from typing import Optional

from slack_sdk import WebClient
from slack_sdk.errors import SlackApiError
from slack_sdk.web.slack_response import SlackResponse

from hartware_lib.exceptions.slack import ApiError
from hartware_lib.settings.slack import SlackSettings

logger = logging.getLogger(__name__)


class SlackAdapter:
    def __init__(self, settings: SlackSettings):
        self.settings = settings
        self.client = WebClient(token=self.settings.api_token)

    def send(self, message: str, channel: Optional[str] = None) -> SlackResponse:
        try:
            return self.client.chat_postMessage(
                channel=channel or self.settings.default_channel, text=message
            )
        except SlackApiError:
            api_error = ApiError()
            logger.warning(api_error.message)

            raise api_error



from typing import Dict, Optional

from aiohttp import ClientResponse, ClientSession


class HttpAsyncAdapter:
    def __init__(self, headers: Optional[Dict] = None):
        self.headers = headers or {}

    async def request(
        self, method: str, url: str, extra_headers: Optional[Dict] = None, **kwargs
    ) -> ClientResponse:
        headers = self.headers

        if extra_headers:
            headers.update(extra_headers)

        async with ClientSession() as session:
            response = await session.request(method, url, headers=headers, **kwargs)

            response.raise_for_status()

            return response

    async def get_text(self, *args, **kwargs) -> str:
        response = await self.request(*args, **kwargs)

        return await response.text()

    async def get_json(self, *args, **kwargs) -> Dict:
        response = await self.request(*args, **kwargs)

        return await response.json()



from glob import glob
from json import dump, dumps, load, loads
from pathlib import Path
from shutil import rmtree
from typing import Any, List

import aiofiles


def create_dir(dir_path: Path) -> None:
    if not dir_path.exists():
        dir_path.mkdir(parents=True)


def delete_dir(dir_path: Path) -> None:
    if dir_path.exists():
        rmtree(dir_path)


def delete_file(file_path: Path) -> None:
    if file_path.exists():
        file_path.unlink()


def search_files(path_regex: Path = Path("*")) -> List[Path]:
    return [Path(path) for path in glob(str(path_regex), recursive=True)]


def exists(path: Path) -> bool:
    return path.exists()


def read_file(file_path: Path) -> str:
    with open(file_path) as file:
        return file.read()


def write_file(file_path: Path, data: str = "") -> None:
    with open(file_path, "w") as file:
        file.write(data)


def read_json_file(file_path: Path, **kwargs: Any) -> dict:
    with open(file_path) as file:
        return load(file, **kwargs)


def write_json_file(file_path: Path, data: dict, indent=2, **kwargs: Any) -> None:
    with open(file_path, "w") as file:
        dump(data, file, indent=indent, **kwargs)


async def async_read_file(file_path: Path) -> str:
    async with aiofiles.open(file_path) as file:
        return await file.read()


async def async_write_file(file_path: Path, data: str = "") -> None:
    async with aiofiles.open(file_path, "w") as file:
        await file.write(data)
        await file.flush()


async def async_read_json_file(file_path: Path, **kwargs: Any) -> dict:
    async with aiofiles.open(file_path) as file:
        return loads(await file.read(), **kwargs)


async def async_write_json_file(
    file_path: Path, data: dict, indent=2, **kwargs: Any
) -> None:
    async with aiofiles.open(file_path, "w") as file:
        await file.write(dumps(data, indent=indent, **kwargs))
        await file.flush()



from pathlib import Path
from typing import Any

from hartware_lib.utils import filesystem


class FileAdapter:
    def __init__(self, file_path: Path):
        self.file_path = file_path

    def create_parent_dir(self) -> None:
        filesystem.create_dir(self.file_path.parent)

    def delete_parent_dir(self) -> None:
        filesystem.delete_dir(self.file_path.parent)

    def delete(self) -> None:
        filesystem.delete_file(self.file_path)

    def exists(self) -> bool:
        return filesystem.exists(self.file_path)

    def read(self) -> str:
        return filesystem.read_file(self.file_path)

    def save(self, data: str = "") -> None:
        filesystem.write_file(self.file_path, data)

    def read_json(self, **kwargs: Any) -> dict:
        return filesystem.read_json_file(self.file_path, **kwargs)

    def save_json(self, data: dict, **kwargs: Any) -> None:
        filesystem.write_json_file(self.file_path, data, **kwargs)

    async def async_read(self) -> str:
        return await filesystem.async_read_file(self.file_path)

    async def async_save(self, data: str = "") -> None:
        await filesystem.async_write_file(self.file_path, data)

    async def async_read_json(self, **kwargs: Any) -> dict:
        return await filesystem.async_read_json_file(self.file_path, **kwargs)

    async def async_save_json(self, data: dict, **kwargs: Any) -> None:
        await filesystem.async_write_json_file(self.file_path, data, **kwargs)



import argparse

from hartware_lib.adapters.slack import SlackAdapter
from hartware_lib.settings.slack import SlackSettings


def slack_send(command: list = []):
    parser = argparse.ArgumentParser(description="Slack message sender.")
    parser.add_argument("message", help="set the message")
    parser.add_argument("-c", "--channel", help="set the channel")

    args = parser.parse_args(command or None)

    settings = SlackSettings()

    return SlackAdapter(settings).send(message=args.message, channel=args.channel)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hartware_lib',
 'hartware_lib.adapters',
 'hartware_lib.commands',
 'hartware_lib.exceptions',
 'hartware_lib.pydantic',
 'hartware_lib.settings',
 'hartware_lib.utils']

package_data = \
{'': ['*']}

install_requires = \
['requests-mock>=1.9.3,<2.0.0', 'types-requests>=2.27.16,<3.0.0']

extras_require = \
{'all': ['slack-sdk>=3.11.2,<4.0.0',
         'pydantic>=1.9.0,<2.0.0',
         'aiofiles>=0.8.0,<0.9.0',
         'aiohttp>=3.8.3,<4.0.0'],
 'async-file': ['aiofiles>=0.8.0,<0.9.0'],
 'async-http': ['aiohttp>=3.8.3,<4.0.0'],
 'pydantic': ['pydantic>=1.9.0,<2.0.0'],
 'slack': ['slack-sdk>=3.11.2,<4.0.0']}

entry_points = \
{'console_scripts': ['slack_send = hartware_lib.commands.slack:slack_send']}

setup_kwargs = {
    'name': 'hartware-lib',
    'version': '0.3.4',
    'description': 'Core helper lib for Hartware codes.',
    'long_description': '# Hartware Lib\n',
    'author': 'Laurent Arthur',
    'author_email': 'laurent.arthur75@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://gitlab.com/ludwig778/python-lib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pydantic import BaseSettings, Field

from hartware_lib.pydantic.field_types import BooleanFromString


class AppBaseSettings(BaseSettings):
    testing: BooleanFromString = Field(default=False, env="TEST")
    debug: BooleanFromString = Field(default=False, env="DEBUG")



from pathlib import Path
from typing import Any, List

from hartware_lib.utils import filesystem


class DirectoryAdapter:
    def __init__(self, dir_path: Path):
        self.dir_path = dir_path

    def create_dir(self) -> None:
        filesystem.create_dir(self.dir_path)

    def delete_dir(self) -> None:
        filesystem.delete_dir(self.dir_path)

    def delete_file(self, filename: str) -> None:
        filesystem.delete_file(self.dir_path / filename)

    def extend(self, sub_dir_path: Path):
        return DirectoryAdapter(dir_path=self.dir_path / sub_dir_path)

    def search(self, filename_regex: str = "*") -> List[Path]:
        return filesystem.search_files(self.dir_path / "**" / filename_regex)

    def dir_exists(self) -> bool:
        return filesystem.exists(self.dir_path)

    def file_exists(self, filename: str) -> bool:
        return filesystem.exists(self.dir_path / filename)

    def read_file(self, filename: str) -> str:
        return filesystem.read_file(self.dir_path / filename)

    def save_file(self, filename: str, data: str = "") -> None:
        filesystem.write_file(self.dir_path / filename, data)

    def read_json_file(self, filename: str, **kwargs: Any) -> dict:
        return filesystem.read_json_file(self.dir_path / filename, **kwargs)

    def save_json_file(self, filename: str, data: dict, **kwargs: Any) -> None:
        filesystem.write_json_file(self.dir_path / filename, data, **kwargs)

    async def async_read_file(self, filename: str) -> str:
        return await filesystem.async_read_file(self.dir_path / filename)

    async def async_save_file(self, filename: str, data: str = "") -> None:
        await filesystem.async_write_file(self.dir_path / filename, data)

    async def async_read_json_file(self, filename: str, **kwargs: Any) -> dict:
        return await filesystem.async_read_json_file(self.dir_path / filename, **kwargs)

    async def async_save_json_file(
        self, filename: str, data: dict, **kwargs: Any
    ) -> None:
        await filesystem.async_write_json_file(self.dir_path / filename, data, **kwargs)
